#
#       TRI REDUCTION SUB ROUTINE
#
#       one-dimensional spectra reffering an image
#                                               : triap1d.cl
#
#       1997/04/15      K.Kawabata  Ver.1.00
#       1998/04/20      K.Kawabata  Ver.2.00 unification
#       2005/04/07      M.Isogai    Ver.2.10 for iraf v2.12
#
#
#    !!! WARNING: File name must not be included '.fits' !!!
#

procedure triap1d( buf1, buf2, buf5, buf6, buf8, buf3, buf4 )

string buf1 { prompt = "Input image Sp1" }
string buf2 { prompt = "Input image Sp2" }
string buf3 { prompt = "Output image Sp1" }
string buf4 { prompt = "Output image Sp2" }
string buf5 { prompt = "Spectrum No of Sp1" }
string buf6 { prompt = "Spectrum No of Sp2" }
#string buf7 { prompt = "postfit file produced by 'tripostfit'" }
string buf8 { prompt = "y-peak shift file" }


begin

#  file fn_postfit
  file fni1, fni2, fno1, fno2
  file fn_log
  file fn_tmp = "triap1d.tmp"
  int dy, c
  int spec1, spec2
  real y1, y2
  int n_version      # IRAF version (210 or 211)

#  fn_postfit = buf7
  fni1 = buf1
  fni2 = buf2
  fno1 = buf3
  fno2 = buf4
  spec1 = int( buf5 )
  spec2 = int( buf6 )
  fn_log = buf8

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" || buf2 == "V2.12" || buf2 == "V2.13" || buf2 == "V2.14")
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'trired'\n"
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }

#---
  
  if( access( fn_tmp ) )
    imdelete( fn_tmp, ver- )

  if( access( fno1//".fits" ) )
    imdelete( fno1, ver- )
  if( access( fno2//".fits" ) )
    imdelete( fno2, ver- )

  imgets( fni1, 'i_naxis2' )
  dy = int( imgets.value )

  blkavg( fni1, fno1, 1, dy, option="sum" )
  blkavg( fni2, fno2, 1, dy, option="sum" )

  hedit( fno1, "DISPAXIS", "1", add+, delete-, verify-, show-, update+ )
  hedit( fno2, "DISPAXIS", "1", add+, delete-, verify-, show-, update+ )

  if( access( fni1 ) )
    delete( fni1, ver- )
  if( access( fni2 ) )
    delete( fni2, ver- )
#  if( access( fno1 ) )
#    delete( fno1, ver- )
#  if( access( fno2 ) )
#    delete( fno2, ver- )

#  if( n_version == 210 )
#  { 
#    wfits( fni1//".fits", fni1, bscale=1.0, bzero=0.0, long_header-,
#           short_header-, bitpix=-32, scale-, autoscale- )
#    wfits( fni2//".fits", fni2, bscale=1.0, bzero=0.0, long_header-,
#           short_header-, bitpix=-32, scale-, autoscale- )
#    wfits( fno1//".fits", fno1, bscale=1.0, bzero=0.0, long_header-,
#           short_header-, bitpix=-32, scale-, autoscale- )
#    wfits( fno2//".fits", fno2, bscale=1.0, bzero=0.0, long_header-,
#           short_header-, bitpix=-32, scale-, autoscale- )
#  }
#  else
#  {
#    if( n_version == 211 )
#    {
#    wfits( fni1//".fits", fni1, bscale=1.0, bzero=0.0, long_header-,
#           fextn="", short_header-, bitpix=-32, scale-, autoscale- )
#    wfits( fni2//".fits", fni2, bscale=1.0, bzero=0.0, long_header-,
#           fextn="", short_header-, bitpix=-32, scale-, autoscale- )
#    wfits( fno1//".fits", fno1, bscale=1.0, bzero=0.0, long_header-,
#           fextn="", short_header-, bitpix=-32, scale-, autoscale- )
#    wfits( fno2//".fits", fno2, bscale=1.0, bzero=0.0, long_header-,
#           fextn="", short_header-, bitpix=-32, scale-, autoscale- )
#    }
#    else
#    {
#      printf( "IRAF version erorr (%d)\n", n_version )
#      bye
#    }
#  }

execprog1:

  tri1dsub( fni1//".fits", spec1, "none", fno1//".fits", fn_tmp )

  list = fn_tmp
  if( fscan( list, y1 ) != 1 )
  {
    printf( "Exective error in tri1dsub\n" )
    goto execprog1
  }
  delete( fn_tmp, ver- )

execprog2:

  tri1dsub( fni2//".fits", spec2, "none", fno2//".fits", fn_tmp )

  list = fn_tmp
  if( fscan( list, y2 ) != 1 )
  {
    printf( "Exective error in tri1dsub\n" )
    goto execprog2
  }
  delete( fn_tmp, ver- )

#  imdelete( fno1//".fits", ver- )
#  imdelete( fno2//".fits", ver- )

#  if( n_version == 210 )
#  {
#    rfits( fno1, "", fno1, make_image+, long_header-, short_header-,\
#           datatype="real", blank=0., scale-, oldirafname-, offset=0 )
#    rfits( fno2, "", fno2, make_image+, long_header-, short_header-,\
#           datatype="real", blank=0., scale-, oldirafname-, offset=0 )
#  }
#  else
#  {
#    if( n_version == 211 )
#    {
#      rfits( fno1, "", fno1//".fits", make_image+, long_header-, short_header-,\
#             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
#      rfits( fno2, "", fno2//".fits", make_image+, long_header-, short_header-,\
#             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
#    }
#    else
#    {
#      printf( "IRAF version erorr (%d)\n", n_version )
#      bye
#    }
#  }

#  delete( fni1, ver- )
#  delete( fni2, ver- )
#  delete( fno1, ver- )
#  delete( fno2, ver- )

#  tricoord( fno1 )
#  tricoord( fno2 )

  printf( "%s %5.2f   %s %5.2f\n", fni1, y1, fni2, y2 )
  printf( "%s %5.2f   %s %5.2f\n", fni1, y1, fni2, y2, >> fn_log )

end
