#
#       TRI REDUCTION SUB ROUTINE
#
#       Transform and extract two-dimensional spectra (OBJ+FLAT1+FLAT2)
#                                               : triextract.cl
#
#       1997/07/08      K.Kawabata  Ver.0.50  original shift routine
#       1997/11/15      K.Kawabata  Ver.1.00  ignore FLAT's
#       1998/02/06      K.Kawabata  Ver.1.10  tri
#       1998/04/20      K.Kawabata  Ver.2.00  unification
#       1998/05/31      K.Kawabata  Ver.2.10  2HL/2S2 auto-distinction
#       2000/02/29      K.Kawabata  Ver.3.00  for SITe-CCD
#

procedure triextract

begin

  file tmp_list, fn_last
  file fn, fn1, fn2, fnnext
  string fn_stack[500]
  int n_image
  int x, c, i, j
  string trilist[500]
  string shorttyp[500], title[500]
  string exptime[500], diaph[500], filter[500]
  string slit[500]
  string listoflist
  int n_list
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7
  string caldir, rdir
  file fn_def
  file logfn
  int y1, y2, y3, y4
  int y1_2HL, y2_2HL, y3_2HL, y4_2HL
  int y1_2S2, y2_2S2, y3_2S2, y4_2S2
  int ydim

  int flag_erase # 0:keep old images, 1:erase old images
  int flag_inquiry   # 1: 'flag_overwrite' applys to all image
  int flag_overwrite # 0:skip 1:overwrite
  int flag_ver       # 0:no display 1:display

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "tri.tmp.time" ) == 1 )
    delete( "tri.tmp.time", ver- )
  time >> tri.tmp.time
  list = "tri.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "tri.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "----------------------------" )
  print( " scan and extract apertures" )
  print( "----------------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "----------------------------", >> logfn )
  print( " scan and extract apertures", >> logfn )
  print( "----------------------------", >> logfn )
  print( "", >> logfn )

  buf = ""
  listoflist = envget( "_rootlist" )
  caldir = envget( "_caldir" )
  fn_def = caldir//envget( "_defhname" )

  tmp_list = mktemp( "tri.tmp.list." )

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do trigroup" )
    print( "Cannot find "//listoflist//"   Please do trigroup", >> logfn )
    bye
  }

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'triinit' process is incomplete.(_fg_ver) Retry.\n" )
      printf( "'triinit' process is incomplete.(_fg_ver) Retry.\n", >> logfn )
      bye
    }
  }

#----- Replace Mode

  if( envget( "_fg_mode" ) == "1" )
  {
    flag_inquiry = 1
    flag_overwrite = 1
  }
  else
  {
    if( envget( "_fg_mode" ) == "2" )
    {
      flag_inquiry = 1
      flag_overwrite = 0
    }
    else
    {
      if( envget( "_fg_mode" ) == "3" )
      {
        flag_inquiry = 0
        flag_overwrite = 0
      }
      else
      {
        printf( "'triinit' process is incomplete.(_fg_mode) Retry.\n" )
        printf( "'triinit' process is incomplete.(_fg_mode) Retry.\n",\
                >> logfn )
        bye
      }
    }
  }
    
#----- Erase/Reserve old image

  if( envget( "_fg_del2" ) == "1" )
  {
    flag_erase = 1
  }
  else
  {
    if( envget( "_fg_del2" ) == "2" )
    {
      flag_erase = 0
    }
    else
    {
      printf( "'triinit' process is incomplete.(_fg_del2) Retry.\n" )
      printf( "'triinit' process is incomplete.(_fg_del2) Retry.\n", >> logfn )
      bye
    }
  }

#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""
  ydim = 0
  y1 = 0
  y3 = 0

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    printf( "Cannot find '%s'\n", fn_def, >> logfn )
    bye
  }

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "NY2" )
    {
      ydim = int( buf3 )
    }
    if( buf2 == "DY1_2HL" )
    {
      y1_2HL = int( buf3 )
    }
    if( buf2 == "DY2_2HL" )
    {
      y3_2HL = int( buf3 )
    }
    if( buf2 == "DY1_2S2" )
    {
      y1_2S2 = int( buf3 )
    }
    if( buf2 == "DY2_2S2" )
    {
      y3_2S2 = int( buf3 )
    }
  }
  if( (ydim*y1_2HL*y3_2HL*y1_2S2*y3_2S2) == 0 )
  {
    printf( "Read error in '%s' for 'NY2' or 'DY1' or 'DY2'\n", fn_def )
    printf( "Read error in '%s' for 'NY2' or 'DY1' or 'DY2'\n", fn_def,\
            >> logfn )
    bye
  }
  y2_2HL = y1_2HL + ydim - 1
  y4_2HL = y3_2HL + ydim - 1
  y2_2S2 = y1_2S2 + ydim - 1
  y4_2S2 = y3_2S2 + ydim - 1

#----- Scanning the list file

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "TRI" )
    {
      print( "The list-file ", listoflist, " is not produced by TRIGROUP.CL" )
      print( "The list-file ", listoflist, " is not produced by TRIGROUP.CL",\
              >> logfn )
      break
    }
    n_list += 1
    trilist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- main routine

  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] != "DAR" && shorttyp[i] != "BIA" &&\
        shorttyp[i] != "NEA" )
    {
      rdir = caldir

diaphselect:
      if( substr( diaph[i], 1, 3 ) == "2HL" )
      {
        y1 = y1_2HL
        y2 = y2_2HL
        y3 = y3_2HL
        y4 = y4_2HL
      }
      else
      {
        if( substr( diaph[i], 1, 3 ) == "2S2" )
        {
          y1 = y1_2S2
          y2 = y2_2S2
          y3 = y3_2S2
          y4 = y4_2S2
        }
	else
	{
	  printf( "Diaph type error. Skip." )
	  printf( "Diaph type error. Skip.", >> logfn )
	  next
	}
      }

      if( rdir == "none" ){
        printf( "Unknown Diaph type in %s (%s)\n", trilist[i], diaph[i] )
        printf( "Continuing with 2HOLEs mode\n" )
        printf( "Unknown Diaph type in %s (%s)\n", trilist[i], diaph[i],\
                >> logfn )
        printf( "Continuing with 2HOLEs mode\n", >> logfn )
        diaph[i] = "2HL"
	goto diaphselect
      }

      if( flag_ver == 1 )
      {
        printf( "--- OBJECT LIST : %s ---\n", trilist[i] )
        printf( "  refereing calib's in %s\n", rdir )
      }
      printf( "--- OBJECT LIST : %s ---\n", trilist[i], >> logfn )
      printf( "  refereing calib's in %s\n", rdir, >> logfn )

#----- extract OBJECT spectra
 
      sed( 's/fits//', trilist[i], > tmp_list )

      n_image = 0
      list = tmp_list
      while( fscan( list, buf ) != EOF )
      {
        n_image += 1
	fn_stack[n_image] = buf
      }
      delete( tmp_list, ver- )

      for( j=1; j<=n_image; j+=1 )
      {
        fn = fn_stack[j]

        fn1 = fn//"px"
#        fn2 = fn//"ms"
        buf1 = fn//"px.1"
        buf2 = fn//"px.2"
        buf3 = fn//"px.3"
        buf4 = fn//"px.4"

	if( j < n_image )
         fnnext = fn_stack[j+1]//"px.1"
	else
         fnnext = "none"

        if( access( fn1//".fits" ) == 0 ){
          if( flag_ver == 1 )
            printf( "%s--(not exist. skip.)  ", fn1 )
          next
        }

        if( access( buf1//".fits" ) == 1 ){

          if( flag_overwrite == 0 )
          {
            if( fnnext != "none" && access( fnnext//".fits" ) == 0 )
            {
	      printf( "\n%s: Maybe insufficient proccess last time.\n", fn1 )
	      printf( "\n%s: Maybe insufficient proccess last time.\n",\
                      fn1, >> logfn )
              goto inquiry
            }

            if( flag_inquiry == 1 )
            {
              if( flag_ver == 1 )
                printf( "%s--(already exists. skip.)  ", fn1 )
              printf( "%s--(already exists. skip.)  ", fn1, >> logfn )
              next
            }
            else
	    {
              printf( "\n%s already exist.\n", fn1 )
              printf( "\n%s already exist.\n", fn1, >> logfn )
          inquiry:
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> " )
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> ", >> logfn )
              c = scan( buf )
              while( buf != "Y" && buf != "y" && buf != "n" && buf != "N" )
                c = scan( buf )
              printf( "%s\n", buf, >> logfn )
              if( buf == "Y" )
              {
                flag_inquiry = 1
                flag_overwrite = 1
              }
              if( buf == "N" )
              {
                flag_inquiry = 1
                flag_overwrite = 0
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn1 )
                printf( "%s--(already exists. skip.)  ", fn1, >> logfn )
                next
              }
              if( buf == "n" )
              {
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn1 )
                printf( "%s--(already exists. skip.)  ", fn1, >> logfn )
                next
              }
            }
          }
          if( flag_ver == 1 )
            printf( "%s(overwrite)  ", fn1 )
          printf( "%s(overwrite)  ", fn1, >> logfn )

	  if( access( buf1//".fits" ) == 1 )
	    imdelete( buf1, ver- )
	  if( access( buf2//".fits" ) == 1 )
	    imdelete( buf2, ver- )
	  if( access( buf3//".fits" ) == 1 )
	    imdelete( buf3, ver- )
	  if( access( buf4//".fits" ) == 1 )
	    imdelete( buf4, ver- )
        }
        else
        {
          if( flag_ver == 1 )
            printf( "%s  ", fn1 )
          printf( "%s  ", fn1, >> logfn )
        }

        triapext( fn1//".fits", rdir, buf1//".fits", buf2//".fits",\
                  buf3//".fits", buf4//".fits", y1, y2, y3, y4 )

        if( access( buf1//".fits" ) == 0 || access( buf2//".fits" ) == 0\
	 || access( buf3//".fits" ) == 0 || access( buf4//".fits" ) == 0 )
	{
	  printf( "Sp-extraction failed. Abnormal end.\n" )
	  printf( "Sp-extraction failed. Abnormal end.\n", >> logfn )
	  bye
	}

        hedit( buf1, "HISTORY", "Spectrum Extracted: reference in "\
               //rdir, delete-, verify-, show-, update+ )
        hedit( buf2, "HISTORY", "Spectrum Extracted: reference in "\
               //rdir, delete-, verify-, show-, update+ )
        hedit( buf3, "HISTORY", "Spectrum Extracted: reference in "\
               //rdir, delete-, verify-, show-, update+ )
        hedit( buf4, "HISTORY", "Spectrum extracted: reference in "\
               //rdir, delete-, verify-, show-, update+ )

	if( flag_erase == 1 )
          imdelete( fn1, ver- )
      }

      if( flag_ver == 1 )
        printf( "\n" )
      printf( "\n", >> logfn )
    }
  }
 

end
