#
#       TRI REDUCTION SUB ROUTINE
#
#       Summing all UP averaged spectra : triupsum2.cl
#
#       1997/09/08      K.Kawabata Ver.1.00
#       1998/06/28      K.Kawabata Ver.1.10
#
#

procedure triupsum2

begin

  file infn, outfn
  file tmp_list
  int i, c
  int n_up = 4
  int xmedian = 5 # Width of median filter in pixel
#  file fn_nposhwp = "TRI.nposhwp"
  string buf=""

  tmp_list = mktemp( "tri.tmp.list." )

#---- delete old images

  sections( "KAP*.fits", option="fullname", > tmp_list )
  if( sections.nimages > 0 )
  {
    printf( "Detected following old images\n" )
    type( tmp_list )
    printf( "\nErase them? (y/n)" ) 
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "n" )
    {
      delete( tmp_list, ver- )
      bye
    }
    imdelete( "@"//tmp_list, ver- )
    delete( tmp_list, ver- )
  }

#----- read number of position angle of half-wave plate

#  if( access( fn_nposhwp ) == 1 )
#  {
#    list = fn_nposhwp
#    if( fscan( list, n_up ) != 1 )
#    {
#      printf( "Warning: cannot read '%s'\n", fn_nposhwp )
#    }
#  }
  if( n_up < 0 )
  {
    printf( "Input sequencial number of PA of hwp (1-16) >> " )
    c = scan( buf )
    while( int( buf ) < 1 || int( buf ) > 16 )
      c = scan( buf )
    n_up = int( buf )

    if( access( fn_nposhwp ) == 1 )
      delete( fn_nposhwp, ver- )
    printf( "%2d\n", n_up, > fn_nposhwp )
  }

#----- main routine

  for( i=0; i<=(n_up-1); i+=1 )
  {

#----- upper hole, even

   # 2HL
    infn = "TRI.*.2HLUe."//i//".fits"
    outfn = envget( "_up2hlu" )//i//".e"
    sections( infn, option="fullname", >> tmp_list )
    if( sections.nimages > 0 )
    {
      printf( "---- %s ----\n", outfn )
      type( tmp_list )
      imsum( input="@"//tmp_list, output=outfn, title="",\
             pixtype="real", calctype="real", option="sum",\
	     hparams="EXPTIME", low_reject=0, high_reject=0, ver- )
      median( outfn, outfn, xmedian, 1 )
      printf( "\n" )
    }
    delete( tmp_list, ver- )

   # 2S2
    infn = "TRI.*.2S2Ue."//i//".fits"
    outfn = envget( "_up2s2u" )//i//".e"
    sections( infn, option="fullname", >> tmp_list )
    if( sections.nimages > 0 )
    {
      printf( "---- %s ----\n", outfn )
      type( tmp_list )
      imsum( input="@"//tmp_list, output=outfn, title="",\
             pixtype="real", calctype="real", option="sum",\
	     hparams="EXPTIME", low_reject=0, high_reject=0, ver- )
      median( outfn, outfn, xmedian, 1 )
      printf( "\n" )
    }
    delete( tmp_list, ver- )

#----- upper hole, odd

   # 2HL
    infn = "TRI.*.2HLUo."//i//".fits"
    outfn = envget( "_up2hlu" )//i//".o"
    sections( infn, option="fullname", >> tmp_list )
    if( sections.nimages > 0 )
    {
      printf( "---- %s ----\n", outfn )
      type( tmp_list )
      imsum( input="@"//tmp_list, output=outfn, title="",\
             pixtype="real", calctype="real", option="sum",\
	     hparams="EXPTIME", low_reject=0, high_reject=0, ver- )
      median( outfn, outfn, xmedian, 1 )
      printf( "\n" )
    }
    delete( tmp_list, ver- )

   # 2S2
    infn = "TRI.*.2S2Uo."//i//".fits"
    outfn = envget( "_up2s2u" )//i//".o"
    sections( infn, option="fullname", >> tmp_list )
    if( sections.nimages > 0 )
    {
      printf( "---- %s ----\n", outfn )
      type( tmp_list )
      imsum( input="@"//tmp_list, output=outfn, title="",\
             pixtype="real", calctype="real", option="sum",\
	     hparams="EXPTIME", low_reject=0, high_reject=0, ver- )
      median( outfn, outfn, xmedian, 1 )
      printf( "\n" )
    }
    delete( tmp_list, ver- )

#----- lower hole, even

   # 2HL
    infn = "TRI.*.2HLLe."//i//".fits"
    outfn = envget( "_up2hll" )//i//".e"
    sections( infn, option="fullname", >> tmp_list )
    if( sections.nimages > 0 )
    {
      printf( "---- %s ----\n", outfn )
      type( tmp_list )
      imsum( input="@"//tmp_list, output=outfn, title="",\
             pixtype="real", calctype="real", option="sum",\
	     hparams="EXPTIME", low_reject=0, high_reject=0, ver- )
      median( outfn, outfn, xmedian, 1 )
      printf( "\n" )
    }
    delete( tmp_list, ver- )

   # 2S2
    infn = "TRI.*.2S2Le."//i//".fits"
    outfn = envget( "_up2s2l" )//i//".e"
    sections( infn, option="fullname", >> tmp_list )
    if( sections.nimages > 0 )
    {
      printf( "---- %s ----\n", outfn )
      type( tmp_list )
      imsum( input="@"//tmp_list, output=outfn, title="",\
             pixtype="real", calctype="real", option="sum",\
	     hparams="EXPTIME", low_reject=0, high_reject=0, ver- )
      median( outfn, outfn, xmedian, 1 )
      printf( "\n" )
    }
    delete( tmp_list, ver- )

#----- lower hole, odd

   # 2HL
    infn = "TRI.*.2HLLo."//i//".fits"
    outfn = envget( "_up2hll" )//i//".o"
    sections( infn, option="fullname", >> tmp_list )
    if( sections.nimages > 0 )
    {
      printf( "---- %s ----\n", outfn )
      type( tmp_list )
      imsum( input="@"//tmp_list, output=outfn, title="",\
             pixtype="real", calctype="real", option="sum",\
	     hparams="EXPTIME", low_reject=0, high_reject=0, ver- )
      median( outfn, outfn, xmedian, 1 )
      printf( "\n" )
    }
    delete( tmp_list, ver- )

   # 2S2
    infn = "TRI.*.2S2Lo."//i//".fits"
    outfn = envget( "_up2s2l" )//i//".o"
    sections( infn, option="fullname", >> tmp_list )
    if( sections.nimages > 0 )
    {
      printf( "---- %s ----\n", outfn )
      type( tmp_list )
      imsum( input="@"//tmp_list, output=outfn, title="",\
             pixtype="real", calctype="real", option="sum",\
	     hparams="EXPTIME", low_reject=0, high_reject=0, ver- )
      median( outfn, outfn, xmedian, 1 )
      printf( "\n" )
    }
    delete( tmp_list, ver- )

  }

end

