/***********************************************************

    hbs reduction subroutine: hotpxtab.c

                                   1998/04/16  K. Kawabata

    usage: % hotpxtab infile threshold outfile

      infile -- input image file (FITS format)

      threshold -- threshold count(ADU) for definition of hot-pixel 

      outfile -- output list (readable for 'fixpix')

************************************************************/
		  
#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<math.h>
#include"fitsio.h"

#include"tridefine.h"

int readpostfit( char *infile, double coef[][6]  ){

  FILE *fp;
  int i;
  char buf[100];

  if( ( fp = fopen( infile, "rt" ) ) == NULL ){
    return( -1 );
  }

  for( i=1; i<=8; i++ ){
    fgets( buf, 100, fp );
    sscanf( buf, "%lf %lf %lf %lf %lf %lf", &coef[i][0], &coef[i][1],\
	    &coef[i][2], &coef[i][3], &coef[i][4], &coef[i][5] );
  }

  fclose( fp );
  return( 0 );

}

int scanfile( char *infile, float count[][NX0]  ){

  fitsfile *fp;
  int i, anynul, status;
  char buf[100];
  float nulval = 0.;
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, infile, READONLY, &status ) != 0 ){
    printf( "cannot open file '%s' status=%d\n", infile, status );
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX0 || naxis2 != NY0 ){
    printf( "error: definition of NX0 or NY0 in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_read_img( fp, TFLOAT, 1L, npix, &nulval,\
      count[0], &anynul, &status ) != 0 ){
    printf( "cannot read from '%s'\n", infile );
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }

  return( 0 );

}

int main( int argc, char *argv[] ){

  float count1[NY0][NX0];
  int i, j;
  long n=0l;
  char infile[50], outfile[50];
  double thres_hotpix;
  FILE *fp;

  if( argc != 4 ){
    printf( "usage: hbshotpxtab infile threshold outfile\n" );
    exit( -1 );
  }

  sprintf( infile,"%s", argv[1] );
  thres_hotpix = atof( argv[2] );
  strcpy( outfile, argv[3] );

  if( scanfile( infile, count1 ) != 0 ){
    printf( "read error in '%s'\n", infile );
    exit( -1 );
  }

  if( ( fp = fopen( outfile, "wt" ) ) == NULL ){
    printf( "Cannot open '%s'\n", outfile );
    exit( -1 );
  }

  for( i=0; i<=(NY0-1); i++ ){
    for( j=0; j<=(NX0-1); j++ ){
      if( count1[i][j] > thres_hotpix ){
	n++;
	fprintf( fp, "%4d %4d  %4d %4d\n", (j+1), (j+1), (i+1), (i+1) );
      }
    }
  }
  printf( "%ld bad-pixels detected\n", n );
  fclose( fp );

  return( 0 );
}
