/***********************************************************

    hbs reduction subroutine: hbshiftsub.c

        1997/07/08  K. Kawabata
        1997/09/04  K. Kawabata
        1998/02/10  K. Kawabata  accurate count near edge

    usage: % hbsshiftsub infile outfile x-shift y-shift

      infile, outfile  --  FITS formatted image file
      x-shift, y-shift --  float value

************************************************************/
		  
#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<math.h>
#include"fitsio.h"

#include"tridefine.h"

int scanfile( char *infile, float count[][NX2]  ){

  fitsfile *fp;
  int i, anynul, status;
  char buf[100];
  float nulval = 0.;
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, infile, READONLY, &status ) != 0 ){
    printf( "cannot open file '%s' status=%d\n", infile, status );
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX2 || naxis2 != NY2 ){
    printf( "error: definition of NX2 or NY2 in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_read_img( fp, TFLOAT, 1L, npix, &nulval,\
      count[0], &anynul, &status ) != 0 ){
    printf( "cannot read from '%s'\n", infile );
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }

  return( 0 );

}

double realcount1( float c[][NX2], int x, double y_low, double y_high ){

  int iy_low, iy_high;
  int i, j;
  double integc= 0.0;

  if( x < 1 || x > NX2 || y_low >= y_high || ( y_low < 0.5 && y_high < 0.5 ) || ( y_low > ( NY2 + 0.5 ) && y_high > ( NY2 + 0.5 ) ) ){
    integc= 0.0;
  }
  else{
    if( y_low < 0.5 )
      y_low = 0.5;
    if( y_high > ( NY2 + 0.5 ) )
      y_high = NY2 + 0.5;

    iy_low = y_low + 1.4999999999;
    iy_high = y_high - 0.4999999999;

    if( iy_low <= iy_high ){
      for( j=iy_low; j<=iy_high; j++ ){
        integc+= c[j-1][x-1];
      }
    }
    if( ( iy_high - iy_low ) > -2 ){
      if( (iy_low - 2) >= 0 )
        integc+= ( ( ( iy_low - 0.5 ) - y_low ) * c[iy_low-2][x-1] );
      if( iy_high <= ( NY2 - 1 ) )
        integc+= ( ( y_high - ( iy_high + 0.5 ) ) * c[iy_high][x-1] );
    }
    else{
      iy_high = iy_high + 1;
      iy_low = iy_low - 1;
      if( iy_high != iy_low ){
        printf( "Error in realcount. y_low=%6.2f(%3d) y_high=%6.2f(%3d)\n",\
	       y_low, iy_low, y_high, iy_high );
        return( 0.0 );
      }
      integc+= ( ( y_high - y_low ) * c[iy_high-1][x-1] );
    }
  }
  return( integc );
}


double realcount2( float c[][NX2], int y, double x_low, double x_high ){

  int ix_low, ix_high;
  int i, j;
  double count = 0.0;

  if( y < 1 || y > NY2 || x_low >= x_high || ( x_low < 0.5 && x_high < 0.5 ) || ( x_low > ( NX2 + 0.5 ) && x_high > ( NX2 + 0.5 ) ) ){
    count = 0.0;
  }
  else{
    if( x_low < 0.5 )
      x_low = 0.5;
    if( x_high > ( NX2 + 0.5 ) )
      x_high = NX2 + 0.5;

    ix_low = x_low + 1.4999999999;
    ix_high = x_high - 0.4999999999;

    if( ix_low <= ix_high ){
      for( j=ix_low; j<=ix_high; j++ ){
        count += c[y-1][j-1];
      }
    }
    if( ( ix_high - ix_low ) > -2 ){
      if( (ix_low - 2) >= 0 )
	count += ( ( ( ix_low - 0.5 ) - x_low ) * c[y-1][ix_low-2] );
      if( ix_high <= ( NX2 - 1 ) )
	count += ( ( x_high - ( ix_high + 0.5 ) ) * c[y-1][ix_high] );
    }
    else{
      ix_high = ix_high + 1;
      ix_low = ix_low - 1;
      if( ix_high != ix_low ){
	printf( "Error in realcount\n" );
	return( 0.0 );
      }
      count += ( ( x_high - x_low ) * c[y-1][ix_high-1] );
    }
  }
  return( count );
}


void spshiftsub( float count1[][NX2], float count2[][NX2], double sx, double sy ){

  int x, y;
  int i, j, k;
  float count_tmp[NY2][NX2];
  double pos1, pos2;

  if( fabs( sx ) > 0.001 ){
    for( y=1; y<=NY2; y++ ){
      for( x=1; x<=NX2; x++ ){
        pos1 = (double)x - sx - 0.5;
        pos2 = (double)x - sx + 0.5;
        count_tmp[y-1][x-1] = realcount2( count1, y, pos1, pos2 );
      }
    }
  }
  else{
    for( y=1; y<=NY2; y++ ){
      for( x=1; x<=NX2; x++ ){
        count_tmp[y-1][x-1] = count1[y-1][x-1];
      }
    }
  }

  if( fabs( sy ) > 0.001 ){
    for( x=1; x<=NX2; x++ ){
      for( y=1; y<=NY2; y++ ){
        pos1 = (double)y - sy - 0.5;
        pos2 = (double)y - sy + 0.5;
        count2[y-1][x-1] = realcount1( count_tmp, x, pos1, pos2 );
      }
    }
  }
  else{
    for( x=1; x<=NX2; x++ ){
      for( y=1; y<=NY2; y++ ){
        count2[y-1][x-1] = count_tmp[y-1][x-1];
      }
    }
  }

}

int writefile( char *outfile, float count[][NX2] ){

  fitsfile *fp;
  int i, status;
  char buf[100];
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, outfile, READWRITE, &status ) != 0 ){
    printf( "Cannot open file '%s' with READWRITE mode. status=%d\n",\
	    outfile, status );
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX2 ){
    printf( "Error: definition of NX2 in 'hbsdefine.h'\n" );
    return( -1 );
  }
  if( naxis2 != NY2 ){
    printf( "Error: definition of y-dimension in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_write_img( fp, TFLOAT, 1, npix, count, &status ) != 0 ){
    printf( "Error in the function fits_write_img\n" );
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }
  return( 0 );
}

int main( int argc, char *argv[] ){

  float count1[NY2][NX2];
  float count2[NY2][NX2];
  char infile[50], outfile[50];
  double sx, sy;

  if( argc != 5 ){
    printf( "usage: hbsshiftsub infile outfile sx sy\n" );
    exit( -1 );
  }

  sprintf( infile, "%s", argv[1] );
  strcpy( outfile, argv[2] );
  sx = atof( argv[3] );
  sy = atof( argv[4] );

  if( scanfile( infile, count1 ) != 0 ){
    printf( "read error in '%s'\n", infile );
    exit( -1 );
  }

  spshiftsub( count1, count2, sx, sy );

  if( writefile( outfile, count2 ) != 0 ){
    printf( "write failed in '%s'\n", outfile );
    return( 1 );
  }

  return( 0 );
}
